using System;
using System.Collections;
using MFToolkit.Net.Web;
using Microsoft.SPOT;


namespace OfficeAlram
{
    public class AlarmService
    {
        private HttpServer Server { get; set; }
        private MidiPlayer Player { get; set; }
        private ArrayList Schedules { get; set; }
        private AlarmTimer Timer { get; set; }
        private StorageManager Storage { get; set; }



        public AlarmService()
        {
            this.Initialize();
        }


        private void Initialize()
        {
            this.Storage = new StorageManager(@"\SD\");
            this.Schedules = new ArrayList();
            this.Schedules.Add(new Schedule(new TimeSpan(11, 0, 0), "default.mid", ""));
            this.Schedules.Add(new Schedule(new TimeSpan(13, 0, 0), "default.mid", "ɽĻ"));
            this.Schedules.Add(new Schedule(new TimeSpan(19, 0, 0), "default.mid", "Ļ"));

            this.Player = MidiPlayer.GetInstance();
            this.Server = new HttpServer(new HttpHandler() { Player = this.Player, Schedules = this.Schedules, StorageManager = this.Storage });
            this.Timer = new AlarmTimer() { Schedules = this.Schedules };
            this.Timer.AlarmEvent += OnAlarm;
        }



        private void OnAlarm(object sender, Schedule schedule)
        {
            if(this.Storage.FileList.Contains(schedule.AudioFileName))
            {
                this.Player.PlayAudio(this.Storage.FileList[schedule.AudioFileName].ToString());
            }
        }



        public void Start()
        {
            this.Server.Start();
            this.Timer.StartAsync();
        }



        public void Stop()
        {
            this.Server.Stop();
            this.Timer.Stop();
            this.Server = null;
            this.Timer = null;
        }
    }
}
