using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using MFToolkit.Net.Web;
using Microsoft.SPOT;
using System.Collections;
using System.IO;
using System.Reflection;

namespace OfficeAlram
{
    public class HttpHandler : IHttpHandler
    {
        private ArrayList Handler = new ArrayList();

        public MidiPlayer Player { get; set; }
        public ArrayList Schedules { get; set; }
        public StorageManager StorageManager { get; set; }
        public delegate void Func(HttpContext context);



        public HttpHandler()
        {
            this.Initialize();
        }



        private void Initialize()
        {
            this.Handler.Add(new DictionaryEntry("/index.html", new Func(Index)));

            this.Handler.Add(new DictionaryEntry("/filelist", new Func(FileList)));
            this.Handler.Add(new DictionaryEntry("/uploadfile", new Func(UploadFile)));
            this.Handler.Add(new DictionaryEntry("/removefile", new Func(RemoveFile)));
            this.Handler.Add(new DictionaryEntry("/renamefile", new Func(RenameFile)));

            this.Handler.Add(new DictionaryEntry("/schedulelist", new Func(ScheduleList)));
            this.Handler.Add(new DictionaryEntry("/addschedule", new Func(AddSchedule)));
            this.Handler.Add(new DictionaryEntry("/removeschedule", new Func(RemoveSchedule)));

            this.Handler.Add(new DictionaryEntry("/playaudio", new Func(PlayAudio)));
            this.Handler.Add(new DictionaryEntry("/stopplay", new Func(StopPlay)));
            this.Handler.Add(new DictionaryEntry("/setvolume", new Func(SetVolume)));
            this.Handler.Add(new DictionaryEntry("/getvolume", new Func(GetVolume)));

            this.Handler.Add(new DictionaryEntry("/setlocaltime", new Func(SetLocalTime)));
            this.Handler.Add(new DictionaryEntry("/getlocaltime", new Func(GetLocalTime)));
            
            
        }


        private void GetLocalTime(HttpContext context) {
            HttpResponse response = context.Response;
            response.ContentType = "application/json; charset=UTF-8";
            DateTime now = DateTime.Now;

            response.Write("{\"year\":" + now.Year + ", \"month\":" + now.Month + ", \"day\":" + now.Day + ", \"hour\":" + now.Hour + ", \"minute\":" + now.Minute + ", \"second\":" + now.Second + "}");
        }



        private void SetLocalTime(HttpContext context)
        {
            HttpRequest request = context.Request;
            if(request.Params.Keys.Contains("year") && request.Params.Keys.Contains("month")
               && request.Params.Keys.Contains("day") && request.Params.Keys.Contains("hour")
               && request.Params.Keys.Contains("minute") && request.Params.Keys.Contains("second"))
            {
                int year = Int32.Parse(request.Params["year"]);
                int month = Int32.Parse(request.Params["month"]);
                int day = Int32.Parse(request.Params["day"]);
                int hour = Int32.Parse(request.Params["hour"]);
                int minute = Int32.Parse(request.Params["minute"]);
                int second = Int32.Parse(request.Params["second"]);
                
                Microsoft.SPOT.Hardware.Utility.SetLocalTime(new DateTime(year, month, day, hour, minute, second));

                this.ResponseResult(context, true);
            }
        }


        private void GetVolume(HttpContext context)
        {
            HttpResponse response = context.Response;
            response.ContentType = "application/json; charset=UTF-8";

            if (this.Player != null) {
                response.Write("{ \"volume\":" + (int)(this.Player.Volume * 100) + "}");
            } else {
                response.Write("{ \"volume\":0}");
            }
        }


        private void SetVolume(HttpContext context)
        {
            bool result = false;
            if (this.Player != null && context.Request.Params.Keys.Contains("volume")) {
                int volume = Int32.Parse(context.Request.Params["volume"]);
                volume %= 101;
                this.Player.Volume = (double)volume / 100.0;
                result = true;
            }

            this.ResponseResult(context, result);
        }


        private void StopPlay(HttpContext context)
        {
            this.Player.StopPlay();
            this.ResponseResult(context, true);
        }


        private void PlayAudio(HttpContext context)
        {
            if(this.Player != null && context.Request.Params.Keys.Contains("fileName"))
            {
                string fileName = context.Request.Params["fileName"];
                if(this.StorageManager.FileList.Contains(fileName) == true)
                {
                    string filePath = this.StorageManager.FileList[fileName].ToString();
                    new Thread(() => this.Player.PlayAudio(filePath)).Start();
                }
                
            }
            this.ResponseResult(context, true);
        }



        private void RemoveSchedule(HttpContext context)
        {
            if (context.Request.Params.Keys.Contains("time")) {
                string time = context.Request.Params["time"];
                Schedule target = null;
                foreach(Schedule schedule in this.Schedules)
                {
                    string m = (schedule.Time.Minutes < 10 ) ? "0" + schedule.Time.Minutes : schedule.Time.Minutes.ToString();
                    string h = (schedule.Time.Hours < 10) ? "0" + schedule.Time.Hours : schedule.Time.Hours.ToString();

                    string t = h + ":" + m + ":00";
                    if (t == time) { target = schedule; break; }
                }
                if (target != null) { this.Schedules.Remove(target); }
            }
            this.ResponseResult(context, true);
        }



        private void AddSchedule(HttpContext context)
        {
            if (context.Request.Params.Keys.Contains("hour") && context.Request.Params.Keys.Contains("minute")
                && context.Request.Params.Keys.Contains("audio") && context.Request.Params.Keys.Contains("descript")) {
                int hour = Int32.Parse(context.Request.Params["hour"]);
                int minute = Int32.Parse(context.Request.Params["minute"]);
                string audio = context.Request.Params["audio"];
                string desc = context.Request.Params["descript"];
                if(this.StorageManager.FileList.Contains(audio) == false)
                {
                    audio = StorageManager.DefaultAudio;
                }

                TimeSpan time = new TimeSpan(hour, minute, 0);
                Schedule newSchedule = new Schedule(time, audio, desc);
                this.Schedules.Add(newSchedule);
            }
            this.ResponseResult(context, true);
        }



        private void ScheduleList(HttpContext context)
        {
            HttpResponse response = context.Response;
            response.ContentType = "application/json; charset=UTF-8";

            string json = "[";
            foreach(Schedule schedule in this.Schedules)
            {
                json += schedule.ToJson() + ",";

            }
            if (Schedules.Count > 0) { json = json.Substring(0, json.Length - 1); }
            json += "]";

            response.Write(json);
        }



        private void RemoveFile(HttpContext context)
        {
            bool result = false;
            if (context.Request.Params.Keys.Contains("fileName"))
            {
                string audio = context.Request.Params["fileName"];
                if (audio.ToLower() == "default.mid") { this.ResponseResult(context, false); return; }
                result = this.StorageManager.RemoveFile(audio);

                // remove
                foreach(Schedule schedule in this.Schedules)
                {
                    if(schedule.AudioFileName == audio)
                    {
                        schedule.AudioFileName = StorageManager.DefaultAudio;
                    }
                }
            }
            this.ResponseResult(context, result);
        }



        private void UploadFile(HttpContext context)
        {
            bool result = false;
            if(context.Request.MimeContent.Keys.Contains("file") == true)
            {
                string name = context.Request.MimeContent["file"].Headers["Content-Disposition"];
                string delimeter = "filename=\"";
                int index = name.IndexOf(delimeter);
                name = name.Substring(index + delimeter.Length);
                name = name.Substring(0, name.Length - 1);
                byte[] data = context.Request.MimeContent["file"].Content;
                result = this.StorageManager.Save(data, name);
            }
            this.ResponseResult(context, result);
        }



        private void RenameFile(HttpContext context)
        {
            bool result = false;
            if(context.Request.Params.Keys.Contains("source") && context.Request.Params.Keys.Contains("target"))
            {
                result = this.StorageManager.Rename(context.Request.Params["source"], context.Request.Params["target"]);
            }
            this.ResponseResult(context, result);
        }



        private void ResponseResult(HttpContext context, bool result)
        {
            context.Response.ContentType = "application/json; charset=UTF-8";
            context.Response.Write("{ \"result\": \"" + result + "\" }");
        }



        private void FileList(HttpContext context)
        {
            HttpResponse response = context.Response;
            response.ContentType = "application/json; charset=UTF-8";

            string json = "[";
            if (this.StorageManager != null)
            {
                foreach (string fileName in this.StorageManager.FileList.Keys)
                {
                    json += "{\"fileName\":\"" + fileName + "\"},";
                }
                if (this.StorageManager.FileList.Count > 0) { json = json.Substring(0, json.Length - 1); }
            }
            json += "]";

            response.Write(json);
        }




        private Func GetHandler(string url)
        {
            foreach (DictionaryEntry entry in this.Handler) {
                if (entry.Key == url)
                {
                    return entry.Value as Func;
                }
            }
            return null;
        }



        private void Index(HttpContext context)
        {
            string indexHtml = Resource.GetString(Resource.StringResources.index);
            HttpResponse response = context.Response;
            response.ContentType = "text/html; charset=UTF-8";
            response.Write(indexHtml);
        }



        private static string LocalIpAddress() {
            return Microsoft.SPOT.Net.NetworkInformation.NetworkInterface.GetAllNetworkInterfaces()[0].IPAddress;
        }



        public void ProcessRequest(HttpContext context)
        {
            Func handler = this.GetHandler(context.Request.RawUrl);
            if (handler == null) {
                context.Response.HttpStatus = MFToolkit.Net.Web.HttpStatusCode.NotFound;
                return;
            }
            try { handler(context); } catch (Exception) { }
        }
    }
}
