using System;
using System.Collections;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using SecretLabs.NETMF.Hardware.Netduino;


namespace OfficeAlram
{
    public class PiezoSpeaker
    {

        private static PiezoSpeaker _instance;
        private static object _lockObject = new object();
        private Hashtable scale = new Hashtable();
        private int beatsPerMinute = 90;
        private int beatTimeInMilliSeconds = 0;
        private int pauseTimeInMilliSeconds = 0;
        private bool IsBusy = false;
        private object _playLockObject = new object();

        private PiezoSpeaker()
        {
            this.beatTimeInMilliSeconds = 30000 / beatsPerMinute;
            this.pauseTimeInMilliSeconds = (int) (this.beatTimeInMilliSeconds * 0.1);

            scale.Add("c", 1915u);
            scale.Add("d", 1700u);
            scale.Add("e", 1519u);
            scale.Add("f", 1432u);
            scale.Add("g", 1275u);
            scale.Add("a", 1136u);
            scale.Add("b", 1014u);
            scale.Add("C", 956u);
            scale.Add("D", 851u);
            scale.Add("E", 758u);
            scale.Add("h", 0u);
        }


        public static PiezoSpeaker GetInstance()
        {
            lock(_lockObject)
            {
                if(_instance == null) { _instance = new PiezoSpeaker(); }
            }
            return _instance;
        }


        public void Play(string song)
        {
            this.Stop();
            lock(_playLockObject)
            {
                this.IsBusy = true;
                using(PWM speaker = new PWM(Cpu.PWMChannel.PWM_5, 0.0, 0.0, false))
                {
                    try
                    {
                        for(int i = 0; i < song.Length; i += 3)
                        {
                            string note = song.Substring(i, 1);
                            int beatCount = int.Parse(song.Substring(i + 1, 2));
                            uint noteDuration = (uint) this.scale[note];
                            //speaker.
                            speaker.Frequency = 
                            if (this.IsBusy == false) { return; }
                            Thread.Sleep(this.beatTimeInMilliSeconds * beatCount - pauseTimeInMilliSeconds);
                            if (this.IsBusy == false) { return; }
                            speaker.DutyCycle = 0.0;
                            Thread.Sleep(pauseTimeInMilliSeconds);
                            if (this.IsBusy == false) { return; }
                        }
                    } catch(Exception) {}
                }
                this.IsBusy = false;
            }
        }


        public void Stop()
        {
            this.IsBusy = false;
        }
    }
}
