using System;
using System.Collections;
using System.IO;
using Microsoft.SPOT;

namespace OfficeAlram
{
    public class StorageManager
    {
        public Hashtable FileList { get; private set; }
        public string Root { get; private set; }
        public const string DefaultAudio = "default.mid";



        public StorageManager(string root)
        {
            this.FileList = new Hashtable();
            this.Root = root;

            this.UpdateFileList();
        }



        public bool UpdateFileList()
        {
            try { 
                DirectoryInfo rootDirectory = new DirectoryInfo(this.Root);
                var i = 0;
                this.FileList.Clear();

                if (rootDirectory.Exists) {
                    foreach (FileInfo file in rootDirectory.GetFiles()) {
                        this.FileList.Add(file.Name, file.FullName);
                        Debug.Print(file.FullName);
                        i++;
                    }
                }
            } catch (Exception) { return false; }

            return true;
        }



        public bool RemoveFile(string fileName)
        {
            try { 
                if (fileName == DefaultAudio) {  return false; }

                string fullPath = this.FileList[fileName].ToString();
                File.Delete(fullPath);
                this.FileList.Remove(fileName);
            } catch (Exception) { return false; }

            return true;
        }



        public bool Rename(string srcFileName, string target)
        {
            try { 
                if (this.FileList.Contains(srcFileName) == false) { return false; }

                string fullPath = this.FileList[srcFileName].ToString();
                string newPath = fullPath.Substring(0, fullPath.Length - srcFileName.Length);
                newPath += target;
                string temp = this.Root + Guid.NewGuid().ToString();
                File.Move(fullPath, temp);
                File.Move(temp, newPath);

                this.FileList.Remove(srcFileName);
                this.FileList.Add(target, newPath);
            } catch (Exception) { return false; }

            return true;
        }



        public bool Save(byte[] data, string fileName) {
            try
            {
                string temp = this.Root + Guid.NewGuid().ToString();
                if(this.FileList.Contains(fileName))
                {
                    File.Move(this.FileList[fileName].ToString(), temp);
                }
                File.WriteAllBytes(this.Root + fileName, data);
                if(this.FileList.Contains(fileName) == false)
                {
                    this.FileList.Add(fileName, this.Root + fileName);
                }
                File.Delete(temp);
            } catch (Exception) { return false; }

            return true;
        }
    }
}
