using System;
using Microsoft.SPOT;

namespace OfficeAlram
{
    public class VariableLengthDecode
    {
        // also reference:  http://home.roadrunner.com/~jgglatt/tech/midifile/vari.htm  for info //

        public static byte[] ToBytes(byte[] variableDataInput) {
            int offset = 1;
            UInt32 value = variableDataInput[0];
            UInt32 numBytes = 1;

            if (((value & 0x80) != 0)) {
                UInt32 c;
                value &= 0x7F;
                do {
                    c = variableDataInput[offset];
                    value = (value << 7) + (c & 0x7F);
                    ++numBytes;
                    ++offset;
                } while ((c & 0x80) != 0);
            }

            return UintToBytes(value);
        }

        public static UInt32 ToUInt32(byte[] variableDataInput) {
            // thanks to:  http://www.lastrayofhope.com/tag/midi/ //
            //
            int offset = 1;
            UInt32 value = variableDataInput[0];
            UInt32 numBytes = 1;

            if (((value & 0x80) != 0)) {
                UInt32 c;
                value &= 0x7F;
                do {
                    c = variableDataInput[offset];
                    value = (value << 7) + (c & 0x7F);
                    ++numBytes;
                    ++offset;
                } while ((c & 0x80) != 0);
            }

            Debug.Print(value.ToString());

            return value;
        }

        public static UInt32 TwoByteDecodeToUInt32(byte variableDataPositionONE, byte variableDataPositionTWO) {
            // thanks to:  http://www.lastrayofhope.com/tag/midi/ // for original code.   This is modified, taylored by my use //
            //
            byte[] variableDataInput = new byte[2] { 
                //(byte)(0x00),    /// MSB
                //(byte)(0x00),
                (byte)(variableDataPositionONE),
                (byte)(variableDataPositionTWO)   /// LSB
            };

            //Debug.Print("Input hex: " + debugBytes2string(variableDataInput));

            int offset = 1;
            UInt32 value = variableDataInput[0];
            UInt32 numBytes = 1;

            if (((value & 0x80) != 0)) {
                UInt32 c;
                value &= 0x7F;
                do {
                    c = variableDataInput[offset];
                    value = (value << 7) + (c & 0x7F);
                    ++numBytes;
                    ++offset;
                } while ((c & 0x80) != 0);
            }

            //Debug.Print("Shifted result: " + debugBytes2string(UintToBytes(value)));

            return value;
        }

        public static byte[] UintToBytes(uint value) {
            return new byte[4] {    ////  reversed for correct order ////
                (byte)((value >> 24) & 0xFF),
                (byte)((value >> 16) & 0xFF),
                (byte)((value >> 8) & 0xFF),
                (byte)(value & 0xFF) };
        }

        public static int BytesToInt32(byte[] value) {
            return (
                value[0] << 24 |
                value[1] << 16 |
                value[2] << 8 |
                value[3] << 0);
        }

        public static int fourBytesToInt32(byte value1, byte value2, byte value3, byte value4) {
            return (
                value1 << 24 |
                value2 << 16 |
                value3 << 8 |
                value4 << 0);
        }

        public static string debugBytes2string(byte[] tempByte) {
            string temp = "";
            for (int b = 0; b < tempByte.Length; b++) {
                temp += (tempByte[b].ToString());
                if (b != tempByte.Length - 1) {
                    temp += ",";
                }
            }
            return temp;
        }
    }
}
